@@ -205,6 +205,7 @@ class Isometry:
             assert mat.shape == (4, 4) or mat.shape == (3, 4)
             rot_mat = mat[:3, :3]
             if ortho:
+                rot_mat = rot_mat.astype(float)
                 rot_mat = project_orthogonal(rot_mat)
             else:
                 rot_scale = np.diag(rot_mat.T @ rot_mat)
@@ -314,7 +315,16 @@ class Isometry:
 
     @staticmethod
     def random():
+        """ Fully random isometry (rotation covers full space) """
         return Isometry(q=Quaternion.random(), t=np.random.random((3,)))
+    
+    @staticmethod
+    def randn(sigma_degree: float = 0.0, sigma_t: float = 0.0):
+        rand_axis = np.random.randn(3)
+        rand_axis /= np.linalg.norm(rand_axis)
+        rand_angle = np.random.randn() * sigma_degree
+        rand_t = np.random.randn(3) * sigma_t
+        return Isometry.from_axis_angle(axis=rand_axis, degrees=rand_angle, t=rand_t)
 
     def inv(self):
         qinv = self.q.inverse
@@ -333,6 +343,10 @@ class Isometry:
             up = np.asarray(up)
 
         z_dir = target - source
+        if np.linalg.norm(z_dir) < 1e-6:
+            logger.warning(f"source {source} and target {target} are too close, use default z_dir.")
+            z_dir = np.asarray([0.0, 0.0, 1.0])
+
         z_dir /= np.linalg.norm(z_dir)
         if up is None:
             up = np.asarray([0.0, 1.0, 0.0])
@@ -412,6 +426,10 @@ class Isometry:
                 other = other.clone()
             else:
                 other = copy.deepcopy(other)
+            if "OrientedBoundingBox" in str(type(other)):
+                other = other.rotate(self.q.rotation_matrix, center=np.zeros((3, 1)))
+                other.translate(self.t)
+                return other
             return other.transform(self.matrix)
         if hasattr(other, "device"):  # Torch tensor
             th_R, th_t = self.torch_matrices(other.device)
@@ -513,6 +531,36 @@ class ScaledIsometry:
             return self.s * res
 
 
+class BoundingBox:
+
+    VERT_SEQ = np.array([
+        [-0.5, -0.5, -0.5], [-0.5, -0.5, 0.5], [-0.5, 0.5, -0.5], [-0.5, 0.5, 0.5],
+        [0.5, -0.5, -0.5], [0.5, -0.5, 0.5], [0.5, 0.5, -0.5], [0.5, 0.5, 0.5]
+    ])
+
+    def __init__(self, iso: Isometry = None, extent: np.ndarray = None) -> None:
+        if iso is None:
+            iso = Isometry()
+        if extent is None:
+            extent = np.ones((3,))
+        self.iso = iso
+        self.extent = np.asarray(extent)
+
+    @property
+    def vertices(self):
+        return self.iso @ (self.VERT_SEQ * self.extent[np.newaxis, :])
+    
+    def __repr__(self):
+        return f"BoundingBox: iso = {self.iso}, extent = {self.extent}"
+    
+    def inside(self, pos: np.ndarray):
+        pos = np.asarray(pos)
+        return np.all(np.abs(self.iso.inv() @ pos) <= self.extent / 2.0, axis=-1)
+    
+    def scaled(self, factor: float):
+        return BoundingBox(iso=self.iso, extent=self.extent * factor)
+
+
 def _isometry_dot(left, right):
     if isinstance(left, Isometry):
         left_iso, left_scale = left, 1.0
