@@ -3,7 +3,6 @@ Copyright 2022 by Jiahui Huang. All rights reserved.
 This file is part of PyCG toolbox and is released under "MIT License Agreement".
 Please see the LICENSE file that should have been included as part of this package.
 """
-
 import copy
 import numpy as np
 from pycg import o3d
@@ -11,7 +10,7 @@ import matplotlib.colors
 import matplotlib.cm
 import math
 
-from pycg.isometry import Isometry
+from pycg.isometry import Isometry, BoundingBox
 from pycg.color import map_quantized_color
 from pycg.exp import logger
 from pyquaternion import Quaternion
@@ -19,6 +18,30 @@ from pathlib import Path
 from typing import List, Dict, Tuple, Union, Iterable
 
 
+try:
+    from multiprocessing import Process, Queue
+
+    # Get monitor size (for some reason, there's a segfault so I can only handle it this way...)
+    def get_monitor_size(q: Queue):
+        from screeninfo import get_monitors
+        q.put(get_monitors()[0])
+
+    queue = Queue()
+    p = Process(target=get_monitor_size, args=(queue, ),
+                daemon=True, stderr=None, stdout=None)
+    p.start()
+    p.join(timeout=0.5)
+    monitor_res = queue.get_nowait()
+
+    MONITOR_HEIGHT = monitor_res.height
+    MONITOR_WIDTH = monitor_res.width
+
+except Exception:
+
+    logger.warning("Failed to obtain monitor info!")
+    MONITOR_WIDTH = MONITOR_HEIGHT = None
+
+
 # Open3D Stuff
 SERIALIZE_PROTO = {
     "PointCloud": {
@@ -221,7 +244,7 @@ def layout_entities(*identities_groups, gaps_dx=None, gaps_dy=None, gaps_dz=None
 def show_3d(*identities_groups, gaps_dx=None, gaps_dy=None, gaps_dz=None, layout=None, margin=1.0,
             x_labels=None, y_labels=None, z_labels=None, show=True, use_new_api=False, group_names=None, cam_path=None,
             key_bindings=None, separate_windows=True, point_size=5.0, scale=1.0, default_camera_kwargs=None,
-            viewport_shading='LIT', auto_plane=False, up_axis="+Y"):
+            viewport_shading='LIT', auto_plane=False, up_axis="+Y", polyscope: bool = False):
     import pycg.render as render
 
     scenes = []
@@ -242,16 +265,14 @@ def show_3d(*identities_groups, gaps_dx=None, gaps_dy=None, gaps_dz=None, layout
 
         if len(default_camera_kwargs) == 0:
             # Get monitor width so that we can fit in.
-            try:
-                from screeninfo import get_monitors
-                screen_width = get_monitors()[0].width
-                screen_height = get_monitors()[0].height - 128
+            if MONITOR_HEIGHT is not None and MONITOR_WIDTH is not None:
+                screen_width = MONITOR_WIDTH
+                screen_height = MONITOR_HEIGHT - 128
                 window_width = min(screen_width // layout_cols, 1024)
                 window_height = min(screen_height // layout_rows, 768)
                 default_camera_kwargs['w'] = int(window_width / scale)
                 default_camera_kwargs['h'] = int(window_height / scale)
-            except:
-                logger.warning("failed to obtain screen info. Is screeninfo package installed?")
+            else:
                 default_camera_kwargs = {'w': 1024, 'h': 768}
 
     else:
@@ -280,6 +301,7 @@ def show_3d(*identities_groups, gaps_dx=None, gaps_dy=None, gaps_dz=None, layout
         scene_titles.append(gn[0] if gn is not None else None)
 
     if separate_windows and len(scenes) > 1:
+        assert not polyscope, "Polyscope is not supported for multiple windows."
         if show:
             render.vis_manager.reset()
             for scene, scene_title in zip(scenes, scene_titles):
@@ -296,9 +318,12 @@ def show_3d(*identities_groups, gaps_dx=None, gaps_dy=None, gaps_dz=None, layout
     else:
         scene = scenes[0]
         if show:
-            scene.preview(title="View-Util 3D" if scene_titles[0] is None else scene_titles[0],
-                          allow_change_pose=cam_path is not None, add_ruler=False, use_new_api=use_new_api,
-                          key_bindings=key_bindings)
+            if polyscope:
+                scene.preview_polyscope()
+            else:
+                scene.preview(title="View-Util 3D" if scene_titles[0] is None else scene_titles[0],
+                            allow_change_pose=cam_path is not None, add_ruler=False, use_new_api=use_new_api,
+                            key_bindings=key_bindings)
         return scene
 
 
@@ -703,9 +728,9 @@ def pointcloud(pc, cid: np.ndarray = None, color: np.ndarray = None, ucid: int =
     if color is not None:
         color = ensure_from_torch(color)
         assert color.shape[0] == pc.shape[0], f"Point and color must have same size {color.shape[0]}, {pc.shape[0]}"
-        if color.dtype == np.uint8:
+        if issubclass(color.dtype.type, np.integer):
             color = color.astype(float) / 255.
-        point_cloud.colors = o3d.utility.Vector3dVector(color[:, :3])
+        point_cloud.colors = o3d.utility.Vector3dVector(color[:, :3].astype(float))
 
     if normal is not None:
         normal = ensure_from_torch(normal)
@@ -989,45 +1014,57 @@ def wireframe(mesh: o3d.geometry.TriangleMesh):
     return geom
 
 
-def wireframe_bbox(extent_min=None, extent_max=None, solid=False, tube=False, tube_radius=0.001,
+def wireframe_bbox(extent_min=None, extent_max=None, 
+                   bbox: BoundingBox = None,
+                   solid=False, tube=False, tube_radius=0.001,
                    cid: Union[np.ndarray, "torch.Tensor"] = None,
                    ucid: int = 0, cmap: str = 'tab10',
                    cfloat: np.ndarray = None, cfloat_cmap: str = 'jet', cfloat_normalize: bool = False,
                    color: Union[np.ndarray, "torch.Tensor"] = None):
-    if extent_min is None:
-        extent_min = [0.0, 0.0, 0.0]
-    if extent_max is None:
-        extent_max = [1.0, 1.0, 1.0]
+    
+    if bbox is None:
+        if extent_min is None:
+            extent_min = [0.0, 0.0, 0.0]
+        if extent_max is None:
+            extent_max = [1.0, 1.0, 1.0]
 
-    if isinstance(extent_min, list):
-        extent_min = np.array(extent_min)
-    if isinstance(extent_max, list):
-        extent_max = np.array(extent_max)
+        if isinstance(extent_min, list):
+            extent_min = np.array(extent_min)
+        if isinstance(extent_max, list):
+            extent_max = np.array(extent_max)
 
-    # Ensure (N, 3) numpy arrays
-    if not isinstance(extent_max[0], Iterable):
-        extent_max = extent_max[None, :]
-    if not isinstance(extent_min[0], Iterable):
-        extent_min = extent_min[None, :]
-    extent_min = ensure_from_torch(extent_min, 2)
-    extent_max = ensure_from_torch(extent_max, 2)
-    assert extent_min.shape[0] == extent_max.shape[0]
+        # Ensure (N, 3) numpy arrays
+        if not isinstance(extent_max[0], Iterable):
+            extent_max = extent_max[None, :]
+        if not isinstance(extent_min[0], Iterable):
+            extent_min = extent_min[None, :]
+        extent_min = ensure_from_torch(extent_min, 2)
+        extent_max = ensure_from_torch(extent_max, 2)
+        assert extent_min.shape[0] == extent_max.shape[0]
+
+        min_x, min_y, min_z = extent_min[:, 0], extent_min[:, 1], extent_min[:, 2]
+        max_x, max_y, max_z = extent_max[:, 0], extent_max[:, 1], extent_max[:, 2]
+        all_points = np.stack([
+            min_x, min_y, min_z, min_x, min_y, max_z, min_x, max_y, min_z, min_x, max_y, max_z,
+            max_x, min_y, min_z, max_x, min_y, max_z, max_x, max_y, min_z, max_x, max_y, max_z
+        ], axis=1).reshape(-1, 3)
+        n_box = extent_min.shape[0]
+
+    else:
+        all_points = bbox.vertices
+        n_box = 1
 
     if cid is not None:
         cid = ensure_from_torch(cid, dim=1).astype(int)
+        ucid = None
 
     if cfloat is not None:
         cfloat = ensure_from_torch(cfloat, dim=1)
+        ucid = None
 
     if color is not None:
         color = ensure_from_torch(color)
-
-    min_x, min_y, min_z = extent_min[:, 0], extent_min[:, 1], extent_min[:, 2]
-    max_x, max_y, max_z = extent_max[:, 0], extent_max[:, 1], extent_max[:, 2]
-    all_points = np.stack([
-        min_x, min_y, min_z, min_x, min_y, max_z, min_x, max_y, min_z, min_x, max_y, max_z,
-        max_x, min_y, min_z, max_x, min_y, max_z, max_x, max_y, min_z, max_x, max_y, max_z
-    ], axis=1).reshape(-1, 3)
+        ucid = None
 
     if not solid:
         line_indices = np.asarray([
@@ -1035,7 +1072,7 @@ def wireframe_bbox(extent_min=None, extent_max=None, solid=False, tube=False, tu
             [0, 4], [1, 5], [2, 6], [3, 7],
             [0, 2], [4, 6], [1, 3], [5, 7]
         ])
-        line_indices = (line_indices[None, ...] + (np.arange(extent_min.shape[0]) * 8)[:, None, None]).reshape(-1, 2)
+        line_indices = (line_indices[None, ...] + (np.arange(n_box) * 8)[:, None, None]).reshape(-1, 2)
         geom = lineset(
             all_points, line_indices,
             cid=np.repeat(cid[:, None], repeats=12, axis=1).flatten() if cid is not None else None,
@@ -1055,7 +1092,7 @@ def wireframe_bbox(extent_min=None, extent_max=None, solid=False, tube=False, tu
             [2, 7, 6], [2, 3, 7], [0, 3, 2], [0, 1, 3],
             [7, 1, 5], [3, 1, 7], [2, 6, 0], [0, 6, 4]
         ])
-        cube_indices = (cube_indices[None, ...] + (np.arange(extent_min.shape[0]) * 8)[:, None, None]).reshape(-1, 3)
+        cube_indices = (cube_indices[None, ...] + (np.arange(n_box) * 8)[:, None, None]).reshape(-1, 3)
         geom = mesh(
             all_points, cube_indices,
             cid=np.repeat(cid[:, None], repeats=8, axis=1).flatten() if cid is not None else None,
@@ -1365,7 +1402,7 @@ def from_file(path: str or Path, compute_normal: bool = True, load_obj_textures:
                 o3d_mesh = o3d.geometry.TriangleMesh()
                 o3d_mesh.vertices = o3d.utility.Vector3dVector(np.asarray(comp_trimesh.vertices))
                 o3d_mesh.triangles = o3d.utility.Vector3iVector(np.asarray(comp_trimesh.faces))
-
+                import pdb; pdb.set_trace()
                 assert comp_trimesh.visual.kind == 'texture'
                 if comp_trimesh.visual.uv is not None:
                     tri_uv = comp_trimesh.visual.uv[comp_trimesh.faces.ravel()]
